@@ -1,6 +1,6 @@
 # Orca
 #
-# Copyright 2004-2006 Sun Microsystems Inc.
+# Copyright 2004-2007 Sun Microsystems Inc.
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Library General Public
@@ -21,20 +21,23 @@
 fallback to local settings if the user settings doesn't exist (e.g., in the
 case of gdm) or doesn't have the specified attribute."""
 
-__id__        = "$Id: settings.py 1996 2007-02-11 18:07:28Z joanied $"
-__version__   = "$Revision: 1996 $"
-__date__      = "$Date: 2007-02-11 13:07:28 -0500 (Sun, 11 Feb 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: settings.py 2434 2007-05-29 15:18:37Z richb $"
+__version__   = "$Revision: 2434 $"
+__date__      = "$Date: 2007-05-29 11:18:37 -0400 (Tue, 29 May 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import os
 import re
-import sys
 
 screenWidth = 640
 screenHeight = 480
 tty = 7
 
+# Whether tool tips can be presented.
+#
+canPresentToolTips = False
+
 try:
     # This can fail due to gtk not being available.  We want to
     # be able to recover from that if possible.  The main driver
@@ -56,9 +59,24 @@ try:
     #
     (atom, format, data) = _root_window.property_get("XFree86_VT")
     tty = data[0]
+
+    # The bug that caused gnome-panel to crash is fixed in GTK 2.10.11.
+    minimum_gtk_version = (100000 * 2) + \
+                          (1000 * 10) + \
+                           11
+    current_gtk_version  = (100000 * gtk.gtk_version[0]) + \
+                           (1000 * gtk.gtk_version[1]) + \
+                            gtk.gtk_version[2]
+    canPresentToolTips = (current_gtk_version >= minimum_gtk_version)
 except:
     pass
 
+try:
+    import gconf
+    gconfClient = gconf.client_get_default()
+except:
+    gconfClient = None
+
 import debug
 from acss import ACSS
 from orca_i18n import _           # for gettext support
@@ -106,6 +124,9 @@ userCustomizableSettings = [
     "magTargetDisplay",
     "verbalizePunctuationStyle",
     "showMainWindow",
+    "quitOrcaNoConfirmation",
+    "presentToolTips",
+    "sayAllStyle",
     "keyboardLayout",
     "speakBlankLines",
 ]
@@ -122,6 +143,7 @@ mainWindowModule = "orca_gui_main"
 #
 guiPreferencesModule    = "orca_gui_prefs"
 consolePreferencesModule= "orca_console_prefs"
+appGuiPreferencesModule = "app_gui_prefs"
 
 # The name of the module that hold the user interface for quitting Orca.
 # This module is expected to have the method, showQuitUI, which will
@@ -174,6 +196,12 @@ PUNCTUATION_STYLE_MOST = 1
 PUNCTUATION_STYLE_ALL  = 0
 verbalizePunctuationStyle = PUNCTUATION_STYLE_MOST
 
+# Say All styles (see sayAllStyle).
+#
+SAYALL_STYLE_LINE     = 0
+SAYALL_STYLE_SENTENCE = 1
+sayAllStyle = SAYALL_STYLE_SENTENCE
+
 # The absolue amount to change the speech rate when
 # increasing or decreasing speech.  This is a numerical
 # value that represents an ACSS rate value.
@@ -203,7 +231,7 @@ silenceSpeech           = False
 # Settings that apply to the particular speech engine to
 # use as well details on the default voices to use.
 #
-speechFactoryModules    = ["espeechfactory","gnomespeechfactory"]
+speechFactoryModules    = ["espeechfactory","gnomespeechfactory","speechdispatcherfactory"]
 speechServerFactory     = "gnomespeechfactory"
 speechServerInfo        = None # None means let the factory decide.
 
@@ -235,6 +263,11 @@ enableBrailleGrouping   = False
 #
 enableBrailleMonitor    = False
 
+# Strings used to indicate checkbox/radio button states in braille:
+#
+brailleCheckBoxIndicators = ["< >", "<x>"]
+brailleRadioButtonIndicators = ["& y", "&=y"]
+
 # If True, enable magnification.
 #
 enableMagnifier                  = False
@@ -314,7 +347,7 @@ enableEchoByWord        = False
 # Note that it is allowable for both enableEchoByWord and enableKeyEcho
 # to be True
 #
-enableKeyEcho           = False
+enableKeyEcho           = True
 
 # If True and key echo is enabled, echo Alphanumeric and punctuation keys.
 #
@@ -340,6 +373,15 @@ enableActionKeys        = True
 #
 showMainWindow          = True
 
+# If True, quit Orca without confirmation when the user presses
+# <Orca-modifier>-q.
+#
+quitOrcaNoConfirmation  = False
+
+# Whether the user wants tooltips presented or not.
+#
+presentToolTips = False and canPresentToolTips
+
 # Keyboard layout options (see keyboardLayout).
 #
 GENERAL_KEYBOARD_LAYOUT_DESKTOP = 1
@@ -393,6 +435,7 @@ brailleBindingsMap      = {}
 # objects.
 #
 cacheValues             = True
+cacheDescriptions       = True
 
 # Script developer feature.  If False, no AT-SPI objects
 # will be cached locally.  Helps determine if there might be a
@@ -452,6 +495,38 @@ timeoutCallback         = None # Set by orca.py:init to orca.timeout
 #
 doubleClickTimeout = 0.5
 
+# Obtain/set information regarding whether accessibility is enabled
+# or not.
+#
+def isAccessibilityEnabled():
+    try:
+        return gconfClient.get_bool("/desktop/gnome/interface/accessibility")
+    except:
+        return False
+
+def setAccessibilityEnabled(enable):
+    try:
+        return gconfClient.set_bool("/desktop/gnome/interface/accessibility",
+                                    enable)
+    except:
+        return False
+
+# Obtain/set information regarding whether the gksu keyboard grab is enabled
+# or not.
+#
+def isGKSUGrabDisabled():
+    try:
+        return gconfClient.get_bool("/apps/gksu/disable-grab")
+    except:
+        return False
+
+def setGKSUGrabDisabled(disable):
+    try:
+        return gconfClient.set_bool("/apps/gksu/disable-grab",
+                                    disable)
+    except:
+        return False
+
 # Allow for the customization of key bindings.
 #
 def overrideKeyBindings(script, keyBindings):
@@ -520,21 +595,61 @@ def getScriptModuleName(app):
 
     return app.name
 
-# Note to translators: the regular expression here represents a
-# string to match in the localized application name as seen by
-# at-poke.  For most cases, the application name is the name of
-# the binary used to start the application, but this is an
-# unreliable assumption.  The only reliable way to do the
-# translation is by running the application and then viewing its
-# name in the main window of at-poke.
+# Translators: the regular expression here represents a string to
+# match in the localized application name as seen by at-poke.  For
+# most cases, the application name is the name of the binary used to
+# start the application, but this is an unreliable assumption.  The
+# only reliable way to do the translation is by running the
+# application and then viewing its name in the main window of at-poke.
+# I wish the AT-SPI spec'd this out as machine readable (unlocalized)
+# names, but it's what we're stuck with (unfortunately).
 #
 setScriptMapping(re.compile(_('[\S\s]*StarOffice[\s\S]*')), "StarOffice")
+
+# Translators: see the regular expression note above.  This is for
+# OpenOffice and StarOffice.
+#
 setScriptMapping(re.compile(_('soffice.bin')), "StarOffice")
+
+# Translators: see the regular expression note above.  This is for the
+# Evolution mail application.
+#
 setScriptMapping(re.compile(_('[Ee]volution')), "Evolution")
-setScriptMapping(re.compile(_('Deer Park')), "Mozilla")
+
+# Translators: see the regular expression note above.  This is for the
+# help application (yelp).
+#
 setScriptMapping(re.compile(_('yelp')), "Mozilla")
+
+# Translators: see the regular expression note above.  This is for a
+# version of Mozilla Firefox, which chooses to create strange names
+# for itself at the drop of a hat.
+#
+setScriptMapping(re.compile(_('Deer Park')), "Mozilla")
+
+# Translators: see the regular expression note above.  This is for a
+# version of Mozilla Firefox, which chooses to create strange names
+# for itself at the drop of a hat.
+#
 setScriptMapping(re.compile(_('Bon Echo')), "Mozilla")
+
+# Translators: see the regular expression note above.  This is for a
+# version of Mozilla Firefox, which chooses to create strange names
+# for itself at the drop of a hat.
+#
 setScriptMapping(re.compile(_('Minefield')), "Mozilla")
+
+# Translators: see the regular expression note above.  This is for
+# the Thunderbird e-mail application.
+#
+setScriptMapping(re.compile(_('Mail/News')), "Thunderbird")
+
+# Translators: see the regular expression note above.  This is for
+# gnome_segv2, which calls itself bug-buddy in at-poke.
+#
 setScriptMapping(re.compile(_('bug-buddy')), "gnome_segv2")
+
+# Translators: see the regular expression note above.  This is for
+# the underlying terminal support in gnome-terminal.
+#
 setScriptMapping(re.compile(_('vte')), "gnome-terminal")
-setScriptMapping(re.compile(_('time-admin')), "users-admin")
