@@ -19,10 +19,10 @@
 
 """Speaks information about the current object of interest."""
 
-__id__        = "$Id: where_am_I.py 2296 2007-04-18 16:45:21Z richb $"
-__version__   = "$Revision: 2296 $"
-__date__      = "$Date: 2007-04-18 12:45:21 -0400 (Wed, 18 Apr 2007) $"
-__copyright__ = "Copyright (c) 2005-2006 Sun Microsystems Inc."
+__id__        = "$Id: where_am_I.py 2420 2007-05-23 22:10:42Z richb $"
+__version__   = "$Revision: 2420 $"
+__date__      = "$Date: 2007-05-23 18:10:42 -0400 (Wed, 23 May 2007) $"
+__copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
 import atspi
@@ -33,6 +33,7 @@ import settings
 import speech
 
 from orca_i18n import _ # for gettext support
+from orca_i18n import ngettext  # for ngettext support
 
 class WhereAmI:
 
@@ -126,6 +127,9 @@ class WhereAmI:
         elif role == rolenames.ROLE_PARAGRAPH:
             self._speakParagraph(obj, doubleClick)
 
+        elif role == rolenames.ROLE_ICON:
+            self._speakIconPanel(obj, doubleClick)
+
         else:
             self._speakGenericObject(obj, doubleClick)
 
@@ -571,6 +575,47 @@ class WhereAmI:
 
         self._speakText(obj, doubleClick)
 
+    def _speakIconPanel(self, obj, doubleClick):
+        """Speak the contents of the pane containing this icon. The
+        number of icons in the pane is spoken. Plus the total number of
+        selected icons. Plus the name of each of the selected icons.
+
+        Arguments:
+        - obj: the icon object that currently has focus.
+        """
+
+        utterances = []
+        panel = obj.parent
+        childCount = panel.childCount
+
+        utterances.append(_("Icon panel"))
+        utterances.append(self._getObjLabelAndName(obj))
+        utterances.append(rolenames.getSpeechForRoleName(obj))
+
+        selectedItems = []
+        totalSelectedItems = 0
+        for i in range(0, childCount):
+            child = panel.child(i)
+            if child.state.count(atspi.Accessibility.STATE_SELECTED):
+                totalSelectedItems += 1
+                selectedItems.append(child)
+
+        # Translators: this is a count of the number of selected icons 
+        # and the count of the total number of icons within an icon panel. 
+        # An example of an icon panel is the Nautilus folder view.
+        #
+        itemString = ngettext("%d of %d item selected",
+                              "%d of %d items selected",
+                              childCount) % \
+                              (totalSelectedItems, childCount)
+        utterances.append(itemString)
+
+        if doubleClick:
+            for i in range(0, len(selectedItems)):
+                utterances.append(self._getObjLabelAndName(selectedItems[i]))
+
+        speech.speakUtterances(utterances)
+
     def _speakGenericObject(self, obj, doubleClick):
         """Speak a generic object; one not specifically handled by
         other methods.
@@ -828,6 +873,165 @@ class WhereAmI:
 
         return text
 
+    def _getTextSelection(self, obj):
+        """Get the text selection for the given object.
+
+        Arguments:
+        - obj: the text object to extract the selected text from.
+
+        Returns: the selected text contents plus the start and end 
+        offsets within the text.
+        """
+
+        textContents = ""
+        textObj = obj.text
+        nSelections = textObj.getNSelections()
+        for i in range(0, nSelections):
+            [startOffset, endOffset] = textObj.getSelection(i)
+
+            debug.println(self._debugLevel,
+                "_getTextSelection: selection start=%d, end=%d" % \
+                (startOffset, endOffset))
+
+            selectedText = textObj.getText(startOffset, endOffset)
+            debug.println(self._debugLevel,
+                "_getTextSelection: selected text=<%s>" % selectedText)
+
+            if i > 0:
+                textContents += " "
+            textContents += selectedText
+
+        return [textContents, startOffset, endOffset]
+
+    def _getTextSelections(self, obj, doubleClick):
+        """Get all the text applicable text selections for the given object.
+        If the user doubleclicked, look to see if there are any previous 
+        or next text objects that also have selected text and add in their 
+        text contents.
+
+        Arguments:
+        - obj: the text object to start extracting the selected text from.
+        - doubleClick: True if the user double-clicked the "where am I" key.
+
+        Returns: all the selected text contents plus the start and end
+        offsets within the text for the given object.
+        """
+
+        textContents = ""
+        startOffset = 0
+        endOffset = 0
+        if obj.text.getNSelections() > 0:
+            [textContents, startOffset, endOffset] = \
+                                            self._getTextSelection(obj)
+
+        if doubleClick:
+            current = obj
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                for relation in current.relations:
+                    if relation.getRelationType() == \
+                           atspi.Accessibility.RELATION_FLOWS_FROM:
+                        prevObj = atspi.Accessible.makeAccessible( \
+                                                  relation.getTarget(0))
+                        if prevObj.text.getNSelections() > 0:
+                            [newTextContents, start, end] = \
+                                         self._getTextSelection(prevObj)
+                            textContents = newTextContents + textContents
+                            current = prevObj
+                            morePossibleSelections = True
+                        else:
+                            displayedText = prevObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current = prevObj
+                                morePossibleSelections = True
+                        break
+
+            current = obj
+            morePossibleSelections = True
+            while morePossibleSelections:
+                morePossibleSelections = False
+                for relation in current.relations:
+                    if relation.getRelationType() == \
+                           atspi.Accessibility.RELATION_FLOWS_TO:
+                        nextObj = atspi.Accessible.makeAccessible( \
+                                                  relation.getTarget(0))
+                        if nextObj.text.getNSelections() > 0:
+                            [newTextContents, start, end] = \
+                                         self._getTextSelection(nextObj)
+                            textContents += newTextContents
+                            current = nextObj
+                            morePossibleSelections = True
+                        else:
+                            displayedText = nextObj.text.getText(0, -1)
+                            if len(displayedText) == 0:
+                                current = nextObj
+                                morePossibleSelections = True
+                        break
+
+        return [textContents, startOffset, endOffset]
+
+    def _hasTextSelections(self, obj):
+        """Return an indication of whether this object has selected text.
+        Note that it's possible that this object has no text, but is part
+        of a selected text area. Because of this, we need to check the
+        objects on either side to see if they are none zero length and
+        have text selections.
+
+        Arguments:
+        - obj: the text object to start checking for selected text.
+
+        Returns: an indication of whether this object has selected text,
+        or adjacent text objects have selected text.
+        """
+
+        currentSelected = False
+        otherSelected = False
+        nSelections = obj.text.getNSelections()
+        if nSelections:
+            currentSelected = True
+        else:
+            otherSelected = False
+            displayedText = obj.text.getText(0, -1)
+            if len(displayedText) == 0:
+                current = obj
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    for relation in current.relations:
+                        if relation.getRelationType() == \
+                               atspi.Accessibility.RELATION_FLOWS_FROM:
+                            prevObj = atspi.Accessible.makeAccessible( \
+                                                      relation.getTarget(0))
+                            if prevObj.text.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = prevObj.text.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current = prevObj
+                                    morePossibleSelections = True
+                            break
+
+                current = obj
+                morePossibleSelections = True
+                while morePossibleSelections:
+                    morePossibleSelections = False
+                    for relation in current.relations:
+                        if relation.getRelationType() == \
+                               atspi.Accessibility.RELATION_FLOWS_TO:
+                            nextObj = atspi.Accessible.makeAccessible( \
+                                                      relation.getTarget(0))
+                            if nextObj.text.getNSelections() > 0:
+                                otherSelected = True
+                            else:
+                                displayedText = nextObj.text.getText(0, -1)
+                                if len(displayedText) == 0:
+                                    current = nextObj
+                                    morePossibleSelections = True
+                            break
+
+        return [currentSelected, otherSelected]
+
     def _getTextContents(self, obj, doubleClick):
         """Returns utterences for text.
 
@@ -849,23 +1053,12 @@ class WhereAmI:
             "_getTextContents: caretOffset=%d, nSelections=%d" % \
             (caretOffset, nSelections))
 
-        if nSelections:
+        [current, other] = self._hasTextSelections(obj)
+        if (doubleClick and (current or other)) or \
+           (not doubleClick and current):
             selected = True
-            for i in range(0, nSelections):
-                [startOffset, endOffset] = textObj.getSelection(i)
-
-                debug.println(self._debugLevel,
-                    "_getTextContents: selection start=%d, end=%d" % \
-                    (startOffset, endOffset))
-
-                selectedText = textObj.getText(startOffset, endOffset)
-                debug.println(self._debugLevel,
-                    "_getTextContents: selected text=<%s>" % selectedText)
-
-                if i > 0:
-                    textContents += " "
-                textContents += selectedText
-
+            [textContents, startOffset, endOffset] = \
+                                  self._getTextSelections(obj, doubleClick)
         else:
             # Get the line containing the caret
             #
